﻿# -*- coding:utf-8 -*-
import numpy as np
import pandas as pd
from sklearn.model_selection import train_test_split
from svmutil import *
from math import log
import matplotlib.pyplot as plt


    
            
def split_data(inFile, trainOutFile, testOutFile):
    data = pd.read_csv(inFile)
    y = data['spdw']
    x = data[['word1','word2','word3','word4','word5','word6','word7','word8','word9','word10','word11','word12','word13','word14','word15',
    'word16','word17','word18','word19','word20','word21','word22','word23','word24','word25','word26','word27','word28','word29',
    'word30','word31','word32','word33','word34','word35','word36','word37','word38','word39','word40','word41','word43','word44',
    'word45','word46','word47','word48','word49','word50']]
    y = y.values
    x = x.values
    y = np.array(y)
    y.reshape([len(y),1])
    x = np.array(x)
    x_train, x_test, y_train, y_test = train_test_split(x, y, test_size=0.3)
    y_train = y_train.reshape(len(y_train),1)
    y_test = y_test.reshape(len(y_test),1)
    #print(y_train.shape)
    traindata = np.concatenate((y_train, x_train),axis=1)
    testdata = np.concatenate((y_test, x_test),axis=1)
    np.savetxt(trainOutFile, traindata, delimiter=',')
    np.savetxt(testOutFile, testdata, delimiter=',')
    
def svm_work(trainFile, testFile):

    #从Libsvm格式的已scale的数据文件中提取x和y
    y_train, x_train = svm_read_problem(trainFile)
    y_test, x_test = svm_read_problem(testFile)
    
    problem = svm_problem(y_train, x_train)                     #通过svm_problem为训练集构建problem
    C_array = np.array([2**i for i in range(7,23,2)])                 # C参数的搜索范围
    g_array = np.array([2**i for i in range(-9,1,2)])                 # gamma参数的搜索范围
    
    #生成meshgrid，以备后续画等高线
    CMatrix, gMatrix = np.meshgrid([log(i,2) for i in C_array], [log(i,2) for i in g_array])    
    test_r2_Matrix = np.zeros_like(CMatrix)
    
    # 初始化最佳测试集结果为一个很小的数字
    best_test_r2 = -100
    
    # 调参过程
    for i in range(len(C_array)):
        for j in range(len(g_array)):
            c = C_array[i]
            g = g_array[j]
            print('\n\nCuurent log(C) is {}, current log(g) is {}'.format(log(c,2), log(g,2)))
            para_str = '-s 3 -t 2  -q -m 1000 -c ' + str(c) + ' -g ' + str(g)       #当前参数字符串
            para = svm_parameter(para_str)                                          #通过svm_parameter设定当前参数
            print('Start model estimation')
            model = svm_train(problem, para)                                        #通过svm_train，利用当前参数para对问题problem进行训练
            
            # 通过svm_predict，用训练好的模型对训练集和测试集进行预测，检验模型效果
            y_train_predict, train_acc, useless = svm_predict(y_train, x_train, model)
            y_test_predict, test_acc, useless = svm_predict(y_test, x_test, model)
            
            # 保存当前的测试r2，如果发现更高的测试r2，则更新之，并保存最佳C参数及gamma参数
            # svm_predict输出的第二个结果——test_acc中包括三个元素，分别是：分类正确率，回归mse,回归r2
            test_r2_Matrix[j, i] = test_acc[2]
            if test_acc[2] > best_test_r2:
                best_c, best_g = c, g
                best_test_r2 = test_acc[2]
                print('Best test r2 is {}'.format(best_test_r2))
                
    # 调参完毕，使用最佳C参数和gamma参数重新训练模型
    c, g = best_c, best_g
    para_str = '-s 3 -t 2  -m 1000 -c ' + str(c) + ' -g ' + str(g)
    para = svm_parameter(para_str)
    model = svm_train(problem, para)
    y_train_predict, train_acc, useless = svm_predict(y_train, x_train, model)
    y_test_predict, test_acc, useless = svm_predict(y_test, x_test, model)
    
    # 画图，输出上述grid搜索中，各组（C, gamma）对应的测试集r2
    plt.contourf(CMatrix, gMatrix, test_r2_Matrix, 15, alpha=0.5, cmap=plt.cm.hot)
    C = plt.contour(CMatrix, gMatrix, test_r2_Matrix, 15, colors='black')
    plt.clabel(C, inline = True, fontsize = 10)
    plt.xlabel('log(C)')
    plt.ylabel('log(gamma)')
    plt.show()
    
    
                

if __name__ == "__main__":
    
    ## split data and prepare for libsvm format data
    # inFilePath = r'D:\00-Statistic\45-temp\hs_bd.csv'
    # trainOutFile = r'D:\00-Statistic\45-temp\hs_bd_train.csv'
    # testOutFile = r'D:\00-Statistic\45-temp\hs_bd_test.csv'
    # split_data(inFilePath,  trainOutFile, testOutFile)
    
    #train svm
    trainFile = r'D:\matlab-files\StatisticWork\svm\hs_train_scale.txt'
    testFile = r'D:\matlab-files\StatisticWork\svm\hs_test_scale.txt'
    svm_work(trainFile, testFile)
